var express = require("express");
var hbs = require('express-hbs');
require('handlebars-form-helpers').register(hbs.handlebars);
var nodemailer = require('nodemailer');
var alertmonUtils = require('./alertmonUtils.js');
var fs = require("fs");
var bodyParser = require("body-parser");
var app = express();

// Setup email
var transporter = nodemailer.createTransport({
    service: 'Gmail',
    auth: {
        user: 'alertmonitorfl@gmail.com',
        pass: '6g*hkvVc%91oo3#$'
    }
});
var mailOptions = {
    from: 'Alert Monitor <alertmonitorfl@gmail.com>',
    to: 'jody@kaplon.us,don@gettner.com',
    subject: 'Alert received',
    text: 'test alert'  // Get custom text later on email generation.
};


var file = "./db/alertmon.db";
var exists = fs.existsSync(file);

if(!exists) {
    console.log("Creating DB file.");
    fs.openSync(file, "w");
}

var sqlite3 = require("sqlite3").verbose();
var db = new sqlite3.Database(file);
var pg = require("pg");
var conString = "postgres://alertmon:alertmon@db/alertmon";


app.use(bodyParser.json());                             // Needed for JSON POST requests from Particle Cores.
app.use(bodyParser.urlencoded({ extended: false }));    // Needed for web POST requests from edit form.

// Use `.hbs` for extensions and find partials in `views/partials`.
app.engine('hbs', hbs.express4({
    partialsDir: __dirname + '/views/partials'
}));
app.set('view engine', 'hbs');
app.set('views', __dirname + '/views');

app.get('/', function(req, res){
    var d = new Date();
    console.log("GET /, " + JSON.stringify(d, 4));
    
    pg.connect(conString, function(err, client, done) {
        if(err) {
            return console.error('error fetching client from pool', err);
        }
        var devIndexQry = 
            "select al.coreid, max(co.corename) as corename, max(co.locationdesc) as locationdesc, mpub.maxpub, al.status as maxstatus " + 
            "from alerts al " + 
                "inner join (select coreid, max(published_at) as maxpub from alerts group by coreid) mpub on al.coreid = mpub.coreid and al.published_at = mpub.maxpub " + 
            "left join cores co on al.coreid = co.coreid " + 
            "group by al.coreid, mpub.maxpub, al.status " +
            "order by mpub.maxpub desc, corename asc ";
        client.query(devIndexQry, function(err, result) {
            //call `done()` to release the client back to the pool
            done();

            if(err) {
                return console.error('error running query', err);
            }

            // Loop over elements in rows array, convert ugly UTC times to pretty local times.
            result.rows.forEach(function(row){
                row.maxpubdate = alertmonUtils.getLocDateFromUTC(row.maxpub);
                row.maxpubtime = alertmonUtils.getLocTimeFromUTC(row.maxpub);

                if(row.maxstatus.toLowerCase().indexOf('alert') > -1){
                    row.rowclass = 'alert-row';
                } else { row.rowclass = 'non-alert-row'; }
            });

            res.render('index', {cores: result.rows}, function(err, html) {
                if(err !== null) {
                    console.log(err);
                } else {
                    res.send(html);
                }
            });
        });
    });
});

app.get('/core/edit/:id', function(req, res){
    var d = new Date();
    var coreId = req.params.id;
    console.log("GET /core/edit/" + coreId + ", " + JSON.stringify(d, 4));

    pg.connect(conString, function(err, client, done) {
        if(err) {
            return console.error('error fetching client from pool', err);
        }
        client.query("select corename, locationdesc from cores where coreid = ($1);", [coreId], function(err, result) {
            //call `done()` to release the client back to the pool
            done();

            if(err) { return console.error('error running query', err); }
            res.render('core-edit', {values: result.rows}, function(err, html) {
                if(err !== null) {
                    console.log(err);
                } else {
                    res.send(html);
                }
            });
        });
    });
});

app.post('/core/edit/:id', function(req, res){
    var d = new Date();
    var coreId = req.params.id;
    console.log("POST /core/edit/" + coreId + "body: " + JSON.stringify(req.body) + ", " + JSON.stringify(d, 4));
    if (!req.body) {
        return res.sendStatus(400);
    } else {
        pg.connect(conString, function(err, client, done) {
            if(err) { return console.error('error fetching client from pool', err); }
            // check existence of row in Cores table, if not there insert, otherwise update. 
            client.query("SELECT coreid FROM cores WHERE coreid = ($1);", [coreId], function(err, result) {
                done();

                if(err) { return console.error('error running query', err); }
                if(typeof result.rowCount == 0) {
                    client.query(
                        "INSERT INTO cores (coreid, corename, locationdesc) VALUES ($1, $2, $3);",
                        [coreId, req.body.deviceName, req.body.locationDesc]
                    );
                } else {
                    if (req.body.deviceName !== "") {
                        client.query("UPDATE cores SET corename =($1) WHERE coreid =($2)", [req.body.deviceName, coreId]);
                    }

                    if (req.body.locationDesc !== "") {
                        client.query("UPDATE cores SET locationdesc =($1) WHERE coreid =($2)", [req.body.locationDesc, coreId]);
                    }
                }
            });
        });
        
        res.redirect(204, 'https://particle.kaplon.us/');
    }
});

app.get('/core/:id', function(req, res){
    var d = new Date();
    var coreId = req.params.id;
    console.log("GET /core/" + coreId + ", " + JSON.stringify(d, 4));

    pg.connect(conString, function(err, client, done) {
        if(err) { return console.error('error fetching client from pool', err); }

        var coreMsgQry = "SELECT al.coreid, al.published_at, al.status, co.corename " +
            "FROM alerts al left join cores co on al.coreid = co.coreid " +
            "WHERE al.coreid = ($1) ORDER BY al.published_at DESC LIMIT 30;"
        client.query(coreMsgQry, [coreId], function(err, result) {
            //call `done()` to release the client back to the pool
            done();

            if(err) {
                return console.error('error running query', err);
            }

            // Loop over elements in rows array, convert ugly UTC times to pretty local times.
            result.rows.forEach(function(row){
                row.pubdate = alertmonUtils.getLocDateFromUTC(row.published_at);
                row.pubtime = alertmonUtils.getLocTimeFromUTC(row.published_at);
                if(row.status.toLowerCase().indexOf('alert') > -1){
                    row.rowclass = 'alert-row';
                } else { row.rowclass = 'non-alert-row'; }
            });

            res.render('core', {alerts: result.rows}, function(err, html) {
                if(err !== null) {
                    console.log(err);
                } else {
                    res.send(html);
                }
            });
        });
    });
});

app.post('/', function(req, res){
    var postEvent = req.body.postEvent;
    var source = req.body.source;
    console.log(req.body);

    var innerDataJSON = JSON.parse(req.body.data);
    var status = JSON.stringify(innerDataJSON.status, null, 4).slice(1,-1);
    var coreid = JSON.stringify(req.body.coreid, null, 4).slice(1,-1);
    var pubAt = JSON.stringify(req.body.published_at, null, 4).slice(1,-1);

    pg.connect(conString, function(err, client, done) {
        if(err) { return console.error('error fetching client from pool', err); }
        client.query(
            "INSERT INTO alerts (origjson, coreid, published_at, status) VALUES ($1, $2, $3, $4);",
            [JSON.stringify(req.body, null, 4), coreid, pubAt, status]
        );
    });

    // Send emails on alerts only
    if(status.toLowerCase().indexOf('alert') > -1){
        mailOptions.text = 'An alert message was received: \n\n';
        mailOptions.text = mailOptions.text + 'Status message, ' + status + '\n';
        mailOptions.text = mailOptions.text + 'Published at, ' + alertmonUtils.getLocDateFromUTC(pubAt) + ' ' + alertmonUtils.getLocTimeFromUTC(pubAt) + '\n';
        //mailOptions.text = mailOptions.text + 'From device, ' + alertmonUtils.getCoreNameFromCoreId(db, coreid) + '\n';
        pg.connect(conString, function(err, client, done) {
            if(err) { return console.error('error fetching client from pool', err); }
            client.query('SELECT corename FROM cores WHERE coreid = $1;', [coreid], function(err, result) {
                if ((err) || (typeof result.rowCount == 0)) {
                    // Don't care about this error or empty result set, still need to send email.
                    result.rows[0].corename = '# No Name #';
                }
                mailOptions.text = mailOptions.text + 'From device, ' + result.rows[0].corename + '\n';

                transporter.sendMail(mailOptions, function(error, info){
                    if(error){
                        console.log(error);
                    }else{
                        console.log('Message sent: ' + info.response);
                    }
                });
            });
        });
    }
    //res.send(JSON.stringify(req.body, null, 4));
    res.status(204).send('POST received');
});

app.listen(3000, function() {
    console.log("Started on PORT 3000");
})
